aboutsummaryrefslogtreecommitdiffstats
path: root/c/wc.c
diff options
context:
space:
mode:
Diffstat (limited to 'c/wc.c')
-rw-r--r--c/wc.c21
1 files changed, 14 insertions, 7 deletions
diff --git a/c/wc.c b/c/wc.c
index 86c3aed..424876d 100644
--- a/c/wc.c
+++ b/c/wc.c
@@ -1,5 +1,5 @@
#include "gbos.h"
/* wc [-l] [-w] [-c]: count lines, words, chars of stdin (default: all). */
void main(void) {
char *argv[8];
unsigned char argc = argv_parse(argv, 8);
@@ -7,17 +7,24 @@ void main(void) {
unsigned char ww = hasflag(argv, argc, 'w');
unsigned char wch = hasflag(argv, argc, 'c');
unsigned int lines = 0, words = 0, chars = 0;
- unsigned char inword = 0, first = 1;
- char c;
+ unsigned char inword = 0, first = 1, i, fd = NOFD;
+ char *fname = 0;
+ int ch;
if (!wl && !ww && !wch) { wl = ww = wch = 1; }
+ for (i = 0; i < argc; i++) if (argv[i][0] != '-') { fname = argv[i]; break; }
+ if (fname) {
+ fd = open(fname, O_READ);
+ if (fd == NOFD) { puts("wc: no such file"); nl(); return; }
+ }
for (;;) {
- c = readc();
- if (c == EOF) break;
+ if (fd == NOFD) { char c = readc(); if (c == EOF) break; ch = c; }
+ else { ch = fgetc(fd); if (ch < 0) break; }
chars++;
- if (c == '\n') lines++;
- if (c == ' ' || c == '\n' || c == '\r' || c == '\t') inword = 0;
+ if (ch == '\n') lines++;
+ if (ch == ' ' || ch == '\n' || ch == '\r' || ch == '\t') inword = 0;
else if (!inword) { inword = 1; words++; }
}
+ if (fd != NOFD) close(fd);
if (wl) { putu(lines); first = 0; }
if (ww) { if (!first) putc(' '); putu(words); first = 0; }
if (wch) { if (!first) putc(' '); putu(chars); }
@@ -1,5 +1,5 @@
#include "gbos.h"
-/* wc [-l] [-w] [-c]: count lines, words, chars of stdin (default: all). */
+/* wc [-l][-w][-c] [file]: count lines/words/chars of a file or stdin. */
void main(void) {
char *argv[8];